/*Data to GPIB-Stream  for using of HPGL-plotter

intializied at  30.04.2020

type  hpglgpib -w example.hpgl 

todo-list
adding -a 0x5  as adress of plotter.
check of  risk by interrupted data stream an point in middle of numbers.



*/

 
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

#include <fcntl.h>

#include <getopt.h>
#include <signal.h>
#include <time.h>


#include "data_to_gpib.h"

int  Dev;



const char *ErrorMnemonic[] = {"EDVR", "ECIC", "ENOL", "EADR", "EARG",
			       "ESAC", "EABO", "ENEB", "EDMA", "",
			       "EOIP", "ECAP", "EFSO", "", "EBUS",
			       "ESTB", "ESRQ", "", "", "", "ETAB"};

static int abort_requested = 0;


static void sigint_handler(int arg)  //?????????????
	{
	abort_requested = 1;
	}

static void GPIBCleanup(int Dev, char* ErrorMsg)
	{
	printf("Error : %s\nibsta = 0x%x iberr = %d (%s)\n",
	       ErrorMsg, ibsta, iberr, ErrorMnemonic[iberr]);
	if (Dev != -1) {
		printf("Cleanup: Taking device offline\n");
		ibonl (Dev, 0);
	}

	}

static struct option long_options[] =
{
	{ "read", required_argument, 0, 'r' },
	{ "write", required_argument, 0, 'w' },
	{ "base", required_argument, 0, 'b' },
	{ "length", required_argument, 0, 'l' },
	{ "debug", no_argument, 0, 'd' },
	{ "flash-id", no_argument, 0, 'i' },
	{ "flash-erase", no_argument, 0, 'e' },
	{ "flash-program", required_argument, 0, 'p' },
	{ "help", no_argument, 0, 'h' },
	{ NULL, 0, 0, 0 }
};

static char* ident = PROG_NAME "   Version: " GIT_VERSION "   Build time: " BUILD_TIME;
static void print_version(void)
{
	fprintf(stderr, "# %s\n", ident);
}


static void usage(void)
{	print_version(); /* XXX should have it's own flag */
	fprintf(stderr, "usage:\n"
		"--read         -r <filename>  read from memory to file\n"
		"--write        -w <filename>  read from file to memory\n"
		"--verify       -v <filename>  verify file and memory\n"
		"--base         -b <base>      base address for read/write/program\n"
		"--length       -l <length>    length of data to be read or written\n"
		"--debug        -d             enable debug logging\n"
		"--flash-id     -i             print ID of flash chips\n"
		"--flash-erase  -e             erase flash at base address\n"
		"--flsh-program -p             program flash at base address\n"
		"");
}





static uint32_t to_number(char *s)
{
	uint32_t val;
	char *endp;

	if (*s == '0' && *(s+1) == 'x')
	{
		val = strtoul(s+2, &endp, 16);
		if (*endp != '\0') {
			fprintf(stderr, "failed to parse: [%s]\n", s);
			return 0;
	}
	}	

	 else
	{
		val = strtoul(s, &endp, 10);
		if (*endp != '\0') {
			fprintf(stderr, "failed to parse: [%s]\n", s);
			return 0;
	}
	}
	return val;
}






int main(int argc, char **argv)
{
	
	uint32_t len, addr, base = 0, length = 0;
	int c;
	uint8_t buf[1024];
	int  optidx;
	FILE *file = NULL;
	int read_op = 0, write_op = 0, identify_flash_op = 0, erase_flash_op = 0, flash_write_op = 0, verify_op =0; //verify_op added
	int readlen;
	time_t start, now;

	while((c = getopt_long(argc, argv, "r:w:b:l:p:v:hied",  //verify  "v" added
			       long_options, &optidx)) != -1)
	 {
		switch(c)
		{
		case 'h':
			usage();
			return 0;
		case 'l':
			if (length)
		{
				fprintf(stderr, "length given twice");
				return 1;
		}
			length = to_number(optarg);
			break;
		case 'b':
			if (base)
				{
				fprintf(stderr, "base given twice");
				return 1;
			}
			base = to_number(optarg);
			break;
		case 'r':
			break;
			
		case 'v':
			break;
		
		case 'w':
			if (file)
			{
				fprintf(stderr, "read given twice");
				return 1;
			}
			file = fopen(optarg, "rb");
			if (!file)
			{
				fprintf(stderr, "failed to open input file: %s\n", strerror(errno));
				return 1;

			}

			write_op = 1;
			break;
		case 'p':
			break;
		case 'i':
			break;
		case 'e':
			break;
		case 'd':
			break;
		}
	}

/*	if (!length)
	{
		fprintf(stderr, "%s: length required\n", __FUNCTION__);
		return 1;
	}
*/
	signal(SIGINT, sigint_handler);  //???????????????????
	

/*device identification - (gpib card, device num, second num, Time out, End of message, disable eos) */
	Dev = ibdev(0, 5, 0, T100s, 1, 0);
	if (ibsta & ERR) {	
		printf("Unable to open device\nibsta = 0x%x iberr = %d", ibsta, iberr);
		return 1;
	}

	ibclr (Dev);
	if (ibsta & ERR) {
		GPIBCleanup(Dev, "Unable to clear device");
		return 1;
		}
	
	readlen = 1;
	time(&start);
	while(readlen)
	{
//		len = MIN(512, base + length - addr);
	 if (write_op)
		{
		readlen = fread(buf, 1, 128, file);
		if (readlen == 0)
			break;
		if (readlen == -1) {
			fprintf(stderr, "read: %s\n", strerror(errno));
			return 1;
		}
		ibwrt (Dev, &buf, readlen);
		if (ibsta & ERR) {
			fprintf(stderr, "%s: writing command failed\n", __FUNCTION__);
			return -1;
			}
			
			
//		addr += readlen;
		
//		if ((addr % 0x100) == 0)
//			time(&now);
//			fprintf(stderr, "\r%08x/%08x, %3d%% %4ds", addr - base, length, ((addr - base) * 100) / length, (int)(now - start));
		}			
	}
	
	fclose(file);
	ibonl(Dev, 0);
	return 0;
}


